Insert, Updates, Deletes — SQLAlchemy 1.0 Documentation

Release: 1.0.15 | Release Date: September 1, 2016

# SQLAlchemy 1.0 Documentation

# Insert, Updates, Deletes

INSERT, UPDATE and DELETE statements build on a hierarchy starting with `UpdateBase`. The `Insert` and `Update` constructs build on the intermediary `ValuesBase`.

`sqlalchemy.sql.expression.` `delete`(table, whereclause=None, bind=None, returning=None, prefixes=None, **dialect_kw)

  Construct `Delete` object.

  Similar functionality is available via the `delete()` method on `Table`.

  Parameters: | table – The table to delete rows from. whereclause – A `ClauseElement` describing the `WHERE` condition of the `DELETE` statement. Note that the `where()` generative method may be used instead.

  See also

  Deletes - SQL Expression Tutorial

`sqlalchemy.sql.expression.` `insert`(table, values=None, inline=False, bind=None, prefixes=None, returning=None, return_defaults=False, **dialect_kw)

  Construct an `Insert` object.

  Similar functionality is available via the `insert()` method on `Table`.

  Parameters: | table – `TableClause` which is the subject of the insert. values – collection of values to be inserted; see `Insert.values()` for a description of allowed formats here. Can be omitted entirely; a `Insert` construct will also dynamically render the VALUES clause at execution time based on the parameters passed to `Connection.execute()`. inline – if True, no attempt will be made to retrieve the SQL-generated default values to be provided within the statement; in particular, this allows SQL expressions to be rendered ‘inline’ within the statement without the need to pre-execute them beforehand; for backends that support “returning”, this turns off the “implicit returning” feature for the statement.

  If both values and compile-time bind parameters are present, the compile-time bind parameters override the information specified within values on a per-key basis.

  The keys within values can be either `Column` objects or their string identifiers. Each key may reference one of:

  - a literal data value (i.e. string, number, etc.);
  - a Column object;
  - a SELECT statement.

  If a `SELECT` statement is specified which references this `INSERT` statement’s table, the statement will be correlated against the `INSERT` statement.

  See also

  Insert Expressions - SQL Expression Tutorial

  Inserts, Updates and Deletes - SQL Expression Tutorial

`sqlalchemy.sql.expression.` `update`(table, whereclause=None, values=None, inline=False, bind=None, prefixes=None, returning=None, return_defaults=False, preserve_parameter_order=False, **dialect_kw)

  Construct an `Update` object.

  E.g.:

      from sqlalchemy import update

      stmt = update(users).where(users.c.id==5).\
              values(name='user #5')

  Similar functionality is available via the `update()` method on `Table`:

      stmt = users.update().\
                  where(users.c.id==5).\
                  values(name='user #5')

  - Parameters: | table – A `Table` object representing the database table to be updated. whereclause – Optional SQL expression describing the `WHERE` condition of the `UPDATE` statement. Modern applications may prefer to use the generative `where()` method to specify the `WHERE` clause. The WHERE clause can refer to multiple tables. For databases which support this, an `UPDATE FROM` clause will be generated, or on MySQL, a multi-table update. The statement will fail on databases that don’t have support for multi-table update statements. A SQL-standard method of referring to additional tables in the WHERE clause is to use a correlated subquery:
        users.update().values(name='ed').where(
                users.c.name==select([addresses.c.email_address]).\
                            where(addresses.c.user_id==users.c.id).\
                            as_scalar()
                )
  Changed in version 0.7.4: The WHERE clause can refer to multiple tables. values – Optional dictionary which specifies the `SET` conditions of the `UPDATE`. If left as `None`, the `SET` conditions are determined from those parameters passed to the statement during the execution and/or compilation of the statement. When compiled standalone without any parameters, the `SET` clause generates for all columns. Modern applications may prefer to use the generative `Update.values()` method to set the values of the UPDATE statement. inline – if True, SQL defaults present on `Column` objects via the `default` keyword will be compiled ‘inline’ into the statement and not pre-executed. This means that their values will not be available in the dictionary returned from `ResultProxy.last_updated_params()`. preserve_parameter_order – if True, the update statement is expected to receive parameters only via the `Update.values()` method, and they must be passed as a Python `list` of 2-tuples. The rendered UPDATE statement will emit the SET clause for each referenced column maintaining this order. New in version 1.0.10. See also Parameter-Ordered Updates - full example of the `preserve_parameter_order` flag

  If both `values` and compile-time bind parameters are present, the compile-time bind parameters override the information specified within `values` on a per-key basis.

  The keys within `values` can be either `Column` objects or their string identifiers (specifically the “key” of the `Column`, normally but not necessarily equivalent to its “name”). Normally, the `Column` objects used here are expected to be part of the target `Table` that is the table to be updated. However when using MySQL, a multiple-table UPDATE statement can refer to columns from any of the tables referred to in the WHERE clause.

  The values referred to in `values` are typically:

  - a literal data value (i.e. string, number, etc.)
  - a SQL expression, such as a related `Column`, a scalar-returning `select()` construct, etc.

  When combining `select()` constructs within the values clause of an `update()` construct, the subquery represented by the `select()` should be correlated to the parent table, that is, providing criterion which links the table inside the subquery to the outer table being updated:

      users.update().values(
              name=select([addresses.c.email_address]).\
                      where(addresses.c.user_id==users.c.id).\
                      as_scalar()
          )

  See also

  Inserts, Updates and Deletes - SQL Expression Language Tutorial

class `sqlalchemy.sql.expression.` `Delete`(table, whereclause=None, bind=None, returning=None, prefixes=None, **dialect_kw)

  Bases: `sqlalchemy.sql.expression.UpdateBase`

  Represent a DELETE construct.

  The `Delete` object is created using the `delete()` function.

  `__init__`(table, whereclause=None, bind=None, returning=None, prefixes=None, **dialect_kw)

    Construct a new `Delete` object.

    This constructor is mirrored as a public API function; see `delete()` for a full usage and argument description.

  `argument_for`(dialect_name, argument_name, default)
    inherited from the `argument_for()` method of `DialectKWArgs`

    Add a new kind of dialect-specific keyword argument for this class.

    E.g.:

        Index.argument_for("mydialect", "length", None)

        some_index = Index('a', 'b', mydialect_length=5)

    The `DialectKWArgs.argument_for()` method is a per-argument way adding extra arguments to the `DefaultDialect.construct_arguments` dictionary. This dictionary provides a list of argument names accepted by various schema-level constructs on behalf of a dialect.

    New dialects should typically specify this dictionary all at once as a data member of the dialect class. The use case for ad-hoc addition of argument names is typically for end-user code that is also using a custom compilation scheme which consumes the additional arguments.

    Parameters: | dialect_name – name of a dialect. The dialect must be locatable, else a `NoSuchModuleError` is raised. The dialect must also include an existing `DefaultDialect.construct_arguments` collection, indicating that it participates in the keyword-argument validation and default system, else `ArgumentError` is raised. If the dialect does not include this collection, then any keyword argument can be specified on behalf of this dialect already. All dialects packaged within SQLAlchemy include this collection, however for third party dialects, support may vary. argument_name – name of the parameter. default – default value of the parameter.

    New in version 0.9.4.

  `bind`
    inherited from the `bind` attribute of `UpdateBase`

    Return a ‘bind’ linked to this `UpdateBase` or a `Table` associated with it.

  `compare`(other, **kw)
    inherited from the `compare()` method of `ClauseElement`

    Compare this ClauseElement to the given ClauseElement.

    Subclasses should override the default behavior, which is a straight identity comparison.

    **kw are arguments consumed by subclass compare() methods and may be used to modify the criteria for comparison. (see `ColumnElement`)

  `compile`(bind=None, dialect=None, **kw)
    inherited from the `compile()` method of `ClauseElement`

    Compile this SQL expression.

    The return value is a `Compiled` object. Calling `str()` or `unicode()` on the returned value will yield a string representation of the result. The `Compiled` object also can return a dictionary of bind parameter names and values using the `params` accessor.

    - Parameters: | bind – An `Engine` or `Connection` from which a `Compiled` will be acquired. This argument takes precedence over this `ClauseElement`‘s bound engine, if any. column_keys – Used for INSERT and UPDATE statements, a list of column names which should be present in the VALUES clause of the compiled statement. If `None`, all columns from the target table object are rendered. dialect – A `Dialect` instance from which a `Compiled` will be acquired. This argument takes precedence over the bind argument as well as this `ClauseElement`‘s bound engine, if any. inline – Used for INSERT statements, for a dialect which does not support inline retrieval of newly generated primary key columns, will force the expression used to create the new primary key value to be rendered inline within the INSERT statement’s VALUES clause. This typically refers to Sequence execution but may also refer to any server-side default generation function associated with a primary key Column. compile_kwargs – optional dictionary of additional parameters that will be passed through to the compiler within all “visit” methods. This allows any custom flag to be passed through to a custom compilation construct, for example. It is also used for the case of passing the `literal_binds` flag through:
          from sqlalchemy.sql import table, column, select

          t = table('t', column('x'))

          s = select([t]).where(t.c.x == 5)

          print s.compile(compile_kwargs={"literal_binds": True})
    New in version 0.9.0.

    See also

    How do I render SQL expressions as strings, possibly with bound parameters inlined?

  `dialect_kwargs`
    inherited from the `dialect_kwargs` attribute of `DialectKWArgs`

    A collection of keyword arguments specified as dialect-specific options to this construct.

    The arguments are present here in their original `<dialect>_<kwarg>` format. Only arguments that were actually passed are included; unlike the `DialectKWArgs.dialect_options` collection, which contains all options known by this dialect including defaults.

    The collection is also writable; keys are accepted of the form `<dialect>_<kwarg>` where the value will be assembled into the list of options.

    New in version 0.9.2.

    Changed in version 0.9.4: The `DialectKWArgs.dialect_kwargs` collection is now writable.

    See also

    `DialectKWArgs.dialect_options` - nested dictionary form

  `dialect_options`
    inherited from the `dialect_options` attribute of `DialectKWArgs`

    A collection of keyword arguments specified as dialect-specific options to this construct.

    This is a two-level nested registry, keyed to `<dialect_name>` and `<argument_name>`. For example, the `postgresql_where` argument would be locatable as:

        arg = my_object.dialect_options['postgresql']['where']

    New in version 0.9.2.

    See also

    `DialectKWArgs.dialect_kwargs` - flat dictionary form

  `execute`(*multiparams, **params)
    inherited from the `execute()` method of `Executable`

    Compile and execute this `Executable`.

  `execution_options`(**kw)
    inherited from the `execution_options()` method of `Executable`

    Set non-SQL options for the statement which take effect during execution.

    Execution options can be set on a per-statement or per `Connection` basis. Additionally, the `Engine` and ORM `Query` objects provide access to execution options which they in turn configure upon connections.

    The `execution_options()` method is generative. A new instance of this statement is returned that contains the options:

        statement = select([table.c.x, table.c.y])
        statement = statement.execution_options(autocommit=True)

    Note that only a subset of possible execution options can be applied to a statement - these include “autocommit” and “stream_results”, but not “isolation_level” or “compiled_cache”. See `Connection.execution_options()` for a full list of possible options.

    See also

    `Connection.execution_options()`

    `Query.execution_options()`

  `kwargs`
    inherited from the `kwargs` attribute of `DialectKWArgs`

    A synonym for `DialectKWArgs.dialect_kwargs`.

  `params`(*arg, **kw)
    inherited from the `params()` method of `UpdateBase`

    Set the parameters for the statement.

    This method raises `NotImplementedError` on the base class, and is overridden by `ValuesBase` to provide the SET/VALUES clause of UPDATE and INSERT.

  `prefix_with`(*expr, **kw)
    inherited from the `prefix_with()` method of `HasPrefixes`

    Add one or more expressions following the statement keyword, i.e. SELECT, INSERT, UPDATE, or DELETE. Generative.

    This is used to support backend-specific prefix keywords such as those provided by MySQL.

    E.g.:

        stmt = table.insert().prefix_with("LOW_PRIORITY", dialect="mysql")

    Multiple prefixes can be specified by multiple calls to `prefix_with()`.

    Parameters: | *expr – textual or `ClauseElement` construct which will be rendered following the INSERT, UPDATE, or DELETE keyword. **kw – A single keyword ‘dialect’ is accepted. This is an optional string dialect name which will limit rendering of this prefix to only that dialect.

  `returning`(*cols)
    inherited from the `returning()` method of `UpdateBase`

    Add a RETURNING or equivalent clause to this statement.

    e.g.:

        stmt = table.update().\
                  where(table.c.data == 'value').\
                  values(status='X').\
                  returning(table.c.server_flag,
                            table.c.updated_timestamp)

        for server_flag, updated_timestamp in connection.execute(stmt):
            print(server_flag, updated_timestamp)

    The given collection of column expressions should be derived from the table that is the target of the INSERT, UPDATE, or DELETE. While `Column` objects are typical, the elements can also be expressions:

        stmt = table.insert().returning(
            (table.c.first_name + " " + table.c.last_name).
            label('fullname'))

    Upon compilation, a RETURNING clause, or database equivalent, will be rendered within the statement. For INSERT and UPDATE, the values are the newly inserted/updated values. For DELETE, the values are those of the rows which were deleted.

    Upon execution, the values of the columns to be returned are made available via the result set and can be iterated using `ResultProxy.fetchone()` and similar. For DBAPIs which do not natively support returning values (i.e. cx_oracle), SQLAlchemy will approximate this behavior at the result level so that a reasonable amount of behavioral neutrality is provided.

    Note that not all databases/DBAPIs support RETURNING. For those backends with no support, an exception is raised upon compilation and/or execution. For those who do support it, the functionality across backends varies greatly, including restrictions on executemany() and other statements which return multiple rows. Please read the documentation notes for the database in use in order to determine the availability of RETURNING.

    See also

    `ValuesBase.return_defaults()` - an alternative method tailored towards efficient fetching of server-side defaults and triggers for single-row INSERTs or UPDATEs.

  `scalar`(*multiparams, **params)
    inherited from the `scalar()` method of `Executable`

    Compile and execute this `Executable`, returning the result’s scalar representation.

  `self_group`(against=None)
    inherited from the `self_group()` method of `ClauseElement`

    Apply a ‘grouping’ to this `ClauseElement`.

    This method is overridden by subclasses to return a “grouping” construct, i.e. parenthesis. In particular it’s used by “binary” expressions to provide a grouping around themselves when placed into a larger expression, as well as by `select()` constructs when placed into the FROM clause of another `select()`. (Note that subqueries should be normally created using the `Select.alias()` method, as many platforms require nested SELECT statements to be named).

    As expressions are composed together, the application of `self_group()` is automatic - end-user code should never need to use this method directly. Note that SQLAlchemy’s clause constructs take operator precedence into account - so parenthesis might not be needed, for example, in an expression like `x OR (y AND z)` - AND takes precedence over OR.

    The base `self_group()` method of `ClauseElement` just returns self.

  `unique_params`(*optionaldict, **kwargs)
    inherited from the `unique_params()` method of `ClauseElement`

    Return a copy with `bindparam()` elements replaced.

    Same functionality as `params()`, except adds unique=True to affected bind parameters so that multiple statements can be used.

  `where`(whereclause)

    Add the given WHERE clause to a newly returned delete construct.

  `with_hint`(text, selectable=None, dialect_name='*')
    inherited from the `with_hint()` method of `UpdateBase`

    Add a table hint for a single table to this INSERT/UPDATE/DELETE statement.

    Note

    `UpdateBase.with_hint()` currently applies only to Microsoft SQL Server. For MySQL INSERT/UPDATE/DELETE hints, use `UpdateBase.prefix_with()`.

    The text of the hint is rendered in the appropriate location for the database backend in use, relative to the `Table` that is the subject of this statement, or optionally to that of the given `Table` passed as the `selectable` argument.

    The `dialect_name` option will limit the rendering of a particular hint to a particular backend. Such as, to add a hint that only takes effect for SQL Server:

        mytable.insert().with_hint("WITH (PAGLOCK)", dialect_name="mssql")

    New in version 0.7.6.

    Parameters: | text – Text of the hint. selectable – optional `Table` that specifies an element of the FROM clause within an UPDATE or DELETE to be the subject of the hint - applies only to certain backends. dialect_name – defaults to `*`, if specified as the name of a particular dialect, will apply these hints only when that dialect is in use.

class `sqlalchemy.sql.expression.` `Insert`(table, values=None, inline=False, bind=None, prefixes=None, returning=None, return_defaults=False, **dialect_kw)

  Bases: `sqlalchemy.sql.expression.ValuesBase`

  Represent an INSERT construct.

  The `Insert` object is created using the `insert()` function.

  See also

  Insert Expressions

  `__init__`(table, values=None, inline=False, bind=None, prefixes=None, returning=None, return_defaults=False, **dialect_kw)

    Construct a new `Insert` object.

    This constructor is mirrored as a public API function; see `insert()` for a full usage and argument description.

  `argument_for`(dialect_name, argument_name, default)
    inherited from the `argument_for()` method of `DialectKWArgs`

    Add a new kind of dialect-specific keyword argument for this class.

    E.g.:

        Index.argument_for("mydialect", "length", None)

        some_index = Index('a', 'b', mydialect_length=5)

    The `DialectKWArgs.argument_for()` method is a per-argument way adding extra arguments to the `DefaultDialect.construct_arguments` dictionary. This dictionary provides a list of argument names accepted by various schema-level constructs on behalf of a dialect.

    New dialects should typically specify this dictionary all at once as a data member of the dialect class. The use case for ad-hoc addition of argument names is typically for end-user code that is also using a custom compilation scheme which consumes the additional arguments.

    Parameters: | dialect_name – name of a dialect. The dialect must be locatable, else a `NoSuchModuleError` is raised. The dialect must also include an existing `DefaultDialect.construct_arguments` collection, indicating that it participates in the keyword-argument validation and default system, else `ArgumentError` is raised. If the dialect does not include this collection, then any keyword argument can be specified on behalf of this dialect already. All dialects packaged within SQLAlchemy include this collection, however for third party dialects, support may vary. argument_name – name of the parameter. default – default value of the parameter.

    New in version 0.9.4.

  `bind`
    inherited from the `bind` attribute of `UpdateBase`

    Return a ‘bind’ linked to this `UpdateBase` or a `Table` associated with it.

  `compare`(other, **kw)
    inherited from the `compare()` method of `ClauseElement`

    Compare this ClauseElement to the given ClauseElement.

    Subclasses should override the default behavior, which is a straight identity comparison.

    **kw are arguments consumed by subclass compare() methods and may be used to modify the criteria for comparison. (see `ColumnElement`)

  `compile`(bind=None, dialect=None, **kw)
    inherited from the `compile()` method of `ClauseElement`

    Compile this SQL expression.

    The return value is a `Compiled` object. Calling `str()` or `unicode()` on the returned value will yield a string representation of the result. The `Compiled` object also can return a dictionary of bind parameter names and values using the `params` accessor.

    - Parameters: | bind – An `Engine` or `Connection` from which a `Compiled` will be acquired. This argument takes precedence over this `ClauseElement`‘s bound engine, if any. column_keys – Used for INSERT and UPDATE statements, a list of column names which should be present in the VALUES clause of the compiled statement. If `None`, all columns from the target table object are rendered. dialect – A `Dialect` instance from which a `Compiled` will be acquired. This argument takes precedence over the bind argument as well as this `ClauseElement`‘s bound engine, if any. inline – Used for INSERT statements, for a dialect which does not support inline retrieval of newly generated primary key columns, will force the expression used to create the new primary key value to be rendered inline within the INSERT statement’s VALUES clause. This typically refers to Sequence execution but may also refer to any server-side default generation function associated with a primary key Column. compile_kwargs – optional dictionary of additional parameters that will be passed through to the compiler within all “visit” methods. This allows any custom flag to be passed through to a custom compilation construct, for example. It is also used for the case of passing the `literal_binds` flag through:
          from sqlalchemy.sql import table, column, select

          t = table('t', column('x'))

          s = select([t]).where(t.c.x == 5)

          print s.compile(compile_kwargs={"literal_binds": True})
    New in version 0.9.0.

    See also

    How do I render SQL expressions as strings, possibly with bound parameters inlined?

  `dialect_kwargs`
    inherited from the `dialect_kwargs` attribute of `DialectKWArgs`

    A collection of keyword arguments specified as dialect-specific options to this construct.

    The arguments are present here in their original `<dialect>_<kwarg>` format. Only arguments that were actually passed are included; unlike the `DialectKWArgs.dialect_options` collection, which contains all options known by this dialect including defaults.

    The collection is also writable; keys are accepted of the form `<dialect>_<kwarg>` where the value will be assembled into the list of options.

    New in version 0.9.2.

    Changed in version 0.9.4: The `DialectKWArgs.dialect_kwargs` collection is now writable.

    See also

    `DialectKWArgs.dialect_options` - nested dictionary form

  `dialect_options`
    inherited from the `dialect_options` attribute of `DialectKWArgs`

    A collection of keyword arguments specified as dialect-specific options to this construct.

    This is a two-level nested registry, keyed to `<dialect_name>` and `<argument_name>`. For example, the `postgresql_where` argument would be locatable as:

        arg = my_object.dialect_options['postgresql']['where']

    New in version 0.9.2.

    See also

    `DialectKWArgs.dialect_kwargs` - flat dictionary form

  `execute`(*multiparams, **params)
    inherited from the `execute()` method of `Executable`

    Compile and execute this `Executable`.

  `execution_options`(**kw)
    inherited from the `execution_options()` method of `Executable`

    Set non-SQL options for the statement which take effect during execution.

    Execution options can be set on a per-statement or per `Connection` basis. Additionally, the `Engine` and ORM `Query` objects provide access to execution options which they in turn configure upon connections.

    The `execution_options()` method is generative. A new instance of this statement is returned that contains the options:

        statement = select([table.c.x, table.c.y])
        statement = statement.execution_options(autocommit=True)

    Note that only a subset of possible execution options can be applied to a statement - these include “autocommit” and “stream_results”, but not “isolation_level” or “compiled_cache”. See `Connection.execution_options()` for a full list of possible options.

    See also

    `Connection.execution_options()`

    `Query.execution_options()`

  `from_select`(names, select, include_defaults=True)

    Return a new `Insert` construct which represents an `INSERT...FROM SELECT` statement.

    e.g.:

        sel = select([table1.c.a, table1.c.b]).where(table1.c.c > 5)
        ins = table2.insert().from_select(['a', 'b'], sel)

    Parameters: | names – a sequence of string column names or `Column` objects representing the target columns. select – a `select()` construct, `FromClause` or other construct which resolves into a `FromClause`, such as an ORM `Query` object, etc. The order of columns returned from this FROM clause should correspond to the order of columns sent as the `names` parameter; while this is not checked before passing along to the database, the database would normally raise an exception if these column lists don’t correspond. include_defaults – if True, non-server default values and SQL expressions as specified on `Column` objects (as documented in Column Insert/Update Defaults) not otherwise specified in the list of names will be rendered into the INSERT and SELECT statements, so that these values are also included in the data to be inserted. Note A Python-side default that uses a Python callable function will only be invoked once for the whole statement, and not per row. New in version 1.0.0: - `Insert.from_select()` now renders Python-side and SQL expression column defaults into the SELECT statement for columns otherwise not included in the list of column names.

    Changed in version 1.0.0: an INSERT that uses FROM SELECT implies that the `insert.inline` flag is set to True, indicating that the statement will not attempt to fetch the “last inserted primary key” or other defaults. The statement deals with an arbitrary number of rows, so the `ResultProxy.inserted_primary_key` accessor does not apply.

    New in version 0.8.3.

  `kwargs`
    inherited from the `kwargs` attribute of `DialectKWArgs`

    A synonym for `DialectKWArgs.dialect_kwargs`.

  `params`(*arg, **kw)
    inherited from the `params()` method of `UpdateBase`

    Set the parameters for the statement.

    This method raises `NotImplementedError` on the base class, and is overridden by `ValuesBase` to provide the SET/VALUES clause of UPDATE and INSERT.

  `prefix_with`(*expr, **kw)
    inherited from the `prefix_with()` method of `HasPrefixes`

    Add one or more expressions following the statement keyword, i.e. SELECT, INSERT, UPDATE, or DELETE. Generative.

    This is used to support backend-specific prefix keywords such as those provided by MySQL.

    E.g.:

        stmt = table.insert().prefix_with("LOW_PRIORITY", dialect="mysql")

    Multiple prefixes can be specified by multiple calls to `prefix_with()`.

    Parameters: | *expr – textual or `ClauseElement` construct which will be rendered following the INSERT, UPDATE, or DELETE keyword. **kw – A single keyword ‘dialect’ is accepted. This is an optional string dialect name which will limit rendering of this prefix to only that dialect.

  `return_defaults`(*cols)
    inherited from the `return_defaults()` method of `ValuesBase`

    Make use of a RETURNING clause for the purpose of fetching server-side expressions and defaults.

    E.g.:

        stmt = table.insert().values(data='newdata').return_defaults()

        result = connection.execute(stmt)

        server_created_at = result.returned_defaults['created_at']

    When used against a backend that supports RETURNING, all column values generated by SQL expression or server-side-default will be added to any existing RETURNING clause, provided that `UpdateBase.returning()` is not used simultaneously. The column values will then be available on the result using the `ResultProxy.returned_defaults` accessor as a dictionary, referring to values keyed to the `Column` object as well as its `.key`.

    This method differs from `UpdateBase.returning()` in these ways:

    1. `ValuesBase.return_defaults()` is only intended for use with an INSERT or an UPDATE statement that matches exactly one row. While the RETURNING construct in the general sense supports multiple rows for a multi-row UPDATE or DELETE statement, or for special cases of INSERT that return multiple rows (e.g. INSERT from SELECT, multi-valued VALUES clause), `ValuesBase.return_defaults()` is intended only for an “ORM-style” single-row INSERT/UPDATE statement. The row returned by the statement is also consumed implicitly when `ValuesBase.return_defaults()` is used. By contrast, `UpdateBase.returning()` leaves the RETURNING result-set intact with a collection of any number of rows.
    2. It is compatible with the existing logic to fetch auto-generated primary key values, also known as “implicit returning”. Backends that support RETURNING will automatically make use of RETURNING in order to fetch the value of newly generated primary keys; while the `UpdateBase.returning()` method circumvents this behavior, `ValuesBase.return_defaults()` leaves it intact.
    3. It can be called against any backend. Backends that don’t support RETURNING will skip the usage of the feature, rather than raising an exception. The return value of `ResultProxy.returned_defaults` will be `None`

    `ValuesBase.return_defaults()` is used by the ORM to provide an efficient implementation for the `eager_defaults` feature of `mapper()`.

    Parameters: | cols – optional list of column key names or `Column` objects. If omitted, all column expressions evaluated on the server are added to the returning list.

    New in version 0.9.0.

    See also

    `UpdateBase.returning()`

    `ResultProxy.returned_defaults`

  `returning`(*cols)
    inherited from the `returning()` method of `UpdateBase`

    Add a RETURNING or equivalent clause to this statement.

    e.g.:

        stmt = table.update().\
                  where(table.c.data == 'value').\
                  values(status='X').\
                  returning(table.c.server_flag,
                            table.c.updated_timestamp)

        for server_flag, updated_timestamp in connection.execute(stmt):
            print(server_flag, updated_timestamp)

    The given collection of column expressions should be derived from the table that is the target of the INSERT, UPDATE, or DELETE. While `Column` objects are typical, the elements can also be expressions:

        stmt = table.insert().returning(
            (table.c.first_name + " " + table.c.last_name).
            label('fullname'))

    Upon compilation, a RETURNING clause, or database equivalent, will be rendered within the statement. For INSERT and UPDATE, the values are the newly inserted/updated values. For DELETE, the values are those of the rows which were deleted.

    Upon execution, the values of the columns to be returned are made available via the result set and can be iterated using `ResultProxy.fetchone()` and similar. For DBAPIs which do not natively support returning values (i.e. cx_oracle), SQLAlchemy will approximate this behavior at the result level so that a reasonable amount of behavioral neutrality is provided.

    Note that not all databases/DBAPIs support RETURNING. For those backends with no support, an exception is raised upon compilation and/or execution. For those who do support it, the functionality across backends varies greatly, including restrictions on executemany() and other statements which return multiple rows. Please read the documentation notes for the database in use in order to determine the availability of RETURNING.

    See also

    `ValuesBase.return_defaults()` - an alternative method tailored towards efficient fetching of server-side defaults and triggers for single-row INSERTs or UPDATEs.

  `scalar`(*multiparams, **params)
    inherited from the `scalar()` method of `Executable`

    Compile and execute this `Executable`, returning the result’s scalar representation.

  `self_group`(against=None)
    inherited from the `self_group()` method of `ClauseElement`

    Apply a ‘grouping’ to this `ClauseElement`.

    This method is overridden by subclasses to return a “grouping” construct, i.e. parenthesis. In particular it’s used by “binary” expressions to provide a grouping around themselves when placed into a larger expression, as well as by `select()` constructs when placed into the FROM clause of another `select()`. (Note that subqueries should be normally created using the `Select.alias()` method, as many platforms require nested SELECT statements to be named).

    As expressions are composed together, the application of `self_group()` is automatic - end-user code should never need to use this method directly. Note that SQLAlchemy’s clause constructs take operator precedence into account - so parenthesis might not be needed, for example, in an expression like `x OR (y AND z)` - AND takes precedence over OR.

    The base `self_group()` method of `ClauseElement` just returns self.

  `unique_params`(*optionaldict, **kwargs)
    inherited from the `unique_params()` method of `ClauseElement`

    Return a copy with `bindparam()` elements replaced.

    Same functionality as `params()`, except adds unique=True to affected bind parameters so that multiple statements can be used.

  `values`(*args, **kwargs)
    inherited from the `values()` method of `ValuesBase`

    specify a fixed VALUES clause for an INSERT statement, or the SET clause for an UPDATE.

    Note that the `Insert` and `Update` constructs support per-execution time formatting of the VALUES and/or SET clauses, based on the arguments passed to `Connection.execute()`. However, the `ValuesBase.values()` method can be used to “fix” a particular set of parameters into the statement.

    Multiple calls to `ValuesBase.values()` will produce a new construct, each one with the parameter list modified to include the new parameters sent. In the typical case of a single dictionary of parameters, the newly passed keys will replace the same keys in the previous construct. In the case of a list-based “multiple values” construct, each new list of values is extended onto the existing list of values.

    - Parameters: | **kwargs – key value pairs representing the string key of a `Column` mapped to the value to be rendered into the VALUES or SET clause:
          users.insert().values(name="some name")

          users.update().where(users.c.id==5).values(name="some name")
    - *args – As an alternative to passing key/value parameters, a dictionary, tuple, or list of dictionaries or tuples can be passed as a single positional argument in order to form the VALUES or SET clause of the statement. The forms that are accepted vary based on whether this is an `Insert` or an `Update` construct. For either an `Insert` or `Update` construct, a single dictionary can be passed, which works the same as that of the kwargs form:
          users.insert().values({"name": "some name"})

          users.update().values({"name": "some new name"})
      Also for either form but more typically for the `Insert` construct, a tuple that contains an entry for every column in the table is also accepted:
          users.insert().values((5, "some name"))
      The `Insert` construct also supports being passed a list of dictionaries or full-table-tuples, which on the server will render the less common SQL syntax of “multiple values” - this syntax is supported on backends such as SQLite, Postgresql, MySQL, but not necessarily others:
          users.insert().values([
                              {"name": "some name"},
                              {"name": "some other name"},
                              {"name": "yet another name"},
                          ])
      The above form would render a multiple VALUES statement similar to:
          INSERT INTO users (name) VALUES
                          (:name_1),
                          (:name_2),
                          (:name_3)
    It is essential to note that passing multiple values is NOT the same as using traditional executemany() form. The above syntax is a special syntax not typically used. To emit an INSERT statement against multiple rows, the normal method is to pass a multiple values list to the `Connection.execute()` method, which is supported by all database backends and is generally more efficient for a very large number of parameters. See also Executing Multiple Statements - an introduction to the traditional Core method of multiple parameter set invocation for INSERTs and other statements. Changed in version 1.0.0: an INSERT that uses a multiple-VALUES clause, even a list of length one, implies that the `Insert.inline` flag is set to True, indicating that the statement will not attempt to fetch the “last inserted primary key” or other defaults. The statement deals with an arbitrary number of rows, so the `ResultProxy.inserted_primary_key` accessor does not apply. Changed in version 1.0.0: A multiple-VALUES INSERT now supports columns with Python side default values and callables in the same way as that of an “executemany” style of invocation; the callable is invoked for each row. See Python-side defaults invoked for each row invidually when using a multivalued insert for other details. The `Update` construct supports a special form which is a list of 2-tuples, which when provided must be passed in conjunction with the `preserve_parameter_order` parameter. This form causes the UPDATE statement to render the SET clauses using the order of parameters given to `Update.values()`, rather than the ordering of columns given in the `Table`. New in version 1.0.10: - added support for parameter-ordered UPDATE statements via the `preserve_parameter_order` flag. See also Parameter-Ordered Updates - full example of the `preserve_parameter_order` flag

    See also

    Inserts, Updates and Deletes - SQL Expression Language Tutorial

    `insert()` - produce an `INSERT` statement

    `update()` - produce an `UPDATE` statement

  `with_hint`(text, selectable=None, dialect_name='*')
    inherited from the `with_hint()` method of `UpdateBase`

    Add a table hint for a single table to this INSERT/UPDATE/DELETE statement.

    Note

    `UpdateBase.with_hint()` currently applies only to Microsoft SQL Server. For MySQL INSERT/UPDATE/DELETE hints, use `UpdateBase.prefix_with()`.

    The text of the hint is rendered in the appropriate location for the database backend in use, relative to the `Table` that is the subject of this statement, or optionally to that of the given `Table` passed as the `selectable` argument.

    The `dialect_name` option will limit the rendering of a particular hint to a particular backend. Such as, to add a hint that only takes effect for SQL Server:

        mytable.insert().with_hint("WITH (PAGLOCK)", dialect_name="mssql")

    New in version 0.7.6.

    Parameters: | text – Text of the hint. selectable – optional `Table` that specifies an element of the FROM clause within an UPDATE or DELETE to be the subject of the hint - applies only to certain backends. dialect_name – defaults to `*`, if specified as the name of a particular dialect, will apply these hints only when that dialect is in use.

class `sqlalchemy.sql.expression.` `Update`(table, whereclause=None, values=None, inline=False, bind=None, prefixes=None, returning=None, return_defaults=False, preserve_parameter_order=False, **dialect_kw)

  Bases: `sqlalchemy.sql.expression.ValuesBase`

  Represent an Update construct.

  The `Update` object is created using the `update()` function.

  `__init__`(table, whereclause=None, values=None, inline=False, bind=None, prefixes=None, returning=None, return_defaults=False, preserve_parameter_order=False, **dialect_kw)

    Construct a new `Update` object.

    This constructor is mirrored as a public API function; see `update()` for a full usage and argument description.

  `argument_for`(dialect_name, argument_name, default)
    inherited from the `argument_for()` method of `DialectKWArgs`

    Add a new kind of dialect-specific keyword argument for this class.

    E.g.:

        Index.argument_for("mydialect", "length", None)

        some_index = Index('a', 'b', mydialect_length=5)

    The `DialectKWArgs.argument_for()` method is a per-argument way adding extra arguments to the `DefaultDialect.construct_arguments` dictionary. This dictionary provides a list of argument names accepted by various schema-level constructs on behalf of a dialect.

    New dialects should typically specify this dictionary all at once as a data member of the dialect class. The use case for ad-hoc addition of argument names is typically for end-user code that is also using a custom compilation scheme which consumes the additional arguments.

    Parameters: | dialect_name – name of a dialect. The dialect must be locatable, else a `NoSuchModuleError` is raised. The dialect must also include an existing `DefaultDialect.construct_arguments` collection, indicating that it participates in the keyword-argument validation and default system, else `ArgumentError` is raised. If the dialect does not include this collection, then any keyword argument can be specified on behalf of this dialect already. All dialects packaged within SQLAlchemy include this collection, however for third party dialects, support may vary. argument_name – name of the parameter. default – default value of the parameter.

    New in version 0.9.4.

  `bind`
    inherited from the `bind` attribute of `UpdateBase`

    Return a ‘bind’ linked to this `UpdateBase` or a `Table` associated with it.

  `compare`(other, **kw)
    inherited from the `compare()` method of `ClauseElement`

    Compare this ClauseElement to the given ClauseElement.

    Subclasses should override the default behavior, which is a straight identity comparison.

    **kw are arguments consumed by subclass compare() methods and may be used to modify the criteria for comparison. (see `ColumnElement`)

  `compile`(bind=None, dialect=None, **kw)
    inherited from the `compile()` method of `ClauseElement`

    Compile this SQL expression.

    The return value is a `Compiled` object. Calling `str()` or `unicode()` on the returned value will yield a string representation of the result. The `Compiled` object also can return a dictionary of bind parameter names and values using the `params` accessor.

    - Parameters: | bind – An `Engine` or `Connection` from which a `Compiled` will be acquired. This argument takes precedence over this `ClauseElement`‘s bound engine, if any. column_keys – Used for INSERT and UPDATE statements, a list of column names which should be present in the VALUES clause of the compiled statement. If `None`, all columns from the target table object are rendered. dialect – A `Dialect` instance from which a `Compiled` will be acquired. This argument takes precedence over the bind argument as well as this `ClauseElement`‘s bound engine, if any. inline – Used for INSERT statements, for a dialect which does not support inline retrieval of newly generated primary key columns, will force the expression used to create the new primary key value to be rendered inline within the INSERT statement’s VALUES clause. This typically refers to Sequence execution but may also refer to any server-side default generation function associated with a primary key Column. compile_kwargs – optional dictionary of additional parameters that will be passed through to the compiler within all “visit” methods. This allows any custom flag to be passed through to a custom compilation construct, for example. It is also used for the case of passing the `literal_binds` flag through:
          from sqlalchemy.sql import table, column, select

          t = table('t', column('x'))

          s = select([t]).where(t.c.x == 5)

          print s.compile(compile_kwargs={"literal_binds": True})
    New in version 0.9.0.

    See also

    How do I render SQL expressions as strings, possibly with bound parameters inlined?

  `dialect_kwargs`
    inherited from the `dialect_kwargs` attribute of `DialectKWArgs`

    A collection of keyword arguments specified as dialect-specific options to this construct.

    The arguments are present here in their original `<dialect>_<kwarg>` format. Only arguments that were actually passed are included; unlike the `DialectKWArgs.dialect_options` collection, which contains all options known by this dialect including defaults.

    The collection is also writable; keys are accepted of the form `<dialect>_<kwarg>` where the value will be assembled into the list of options.

    New in version 0.9.2.

    Changed in version 0.9.4: The `DialectKWArgs.dialect_kwargs` collection is now writable.

    See also

    `DialectKWArgs.dialect_options` - nested dictionary form

  `dialect_options`
    inherited from the `dialect_options` attribute of `DialectKWArgs`

    A collection of keyword arguments specified as dialect-specific options to this construct.

    This is a two-level nested registry, keyed to `<dialect_name>` and `<argument_name>`. For example, the `postgresql_where` argument would be locatable as:

        arg = my_object.dialect_options['postgresql']['where']

    New in version 0.9.2.

    See also

    `DialectKWArgs.dialect_kwargs` - flat dictionary form

  `execute`(*multiparams, **params)
    inherited from the `execute()` method of `Executable`

    Compile and execute this `Executable`.

  `execution_options`(**kw)
    inherited from the `execution_options()` method of `Executable`

    Set non-SQL options for the statement which take effect during execution.

    Execution options can be set on a per-statement or per `Connection` basis. Additionally, the `Engine` and ORM `Query` objects provide access to execution options which they in turn configure upon connections.

    The `execution_options()` method is generative. A new instance of this statement is returned that contains the options:

        statement = select([table.c.x, table.c.y])
        statement = statement.execution_options(autocommit=True)

    Note that only a subset of possible execution options can be applied to a statement - these include “autocommit” and “stream_results”, but not “isolation_level” or “compiled_cache”. See `Connection.execution_options()` for a full list of possible options.

    See also

    `Connection.execution_options()`

    `Query.execution_options()`

  `kwargs`
    inherited from the `kwargs` attribute of `DialectKWArgs`

    A synonym for `DialectKWArgs.dialect_kwargs`.

  `params`(*arg, **kw)
    inherited from the `params()` method of `UpdateBase`

    Set the parameters for the statement.

    This method raises `NotImplementedError` on the base class, and is overridden by `ValuesBase` to provide the SET/VALUES clause of UPDATE and INSERT.

  `prefix_with`(*expr, **kw)
    inherited from the `prefix_with()` method of `HasPrefixes`

    Add one or more expressions following the statement keyword, i.e. SELECT, INSERT, UPDATE, or DELETE. Generative.

    This is used to support backend-specific prefix keywords such as those provided by MySQL.

    E.g.:

        stmt = table.insert().prefix_with("LOW_PRIORITY", dialect="mysql")

    Multiple prefixes can be specified by multiple calls to `prefix_with()`.

    Parameters: | *expr – textual or `ClauseElement` construct which will be rendered following the INSERT, UPDATE, or DELETE keyword. **kw – A single keyword ‘dialect’ is accepted. This is an optional string dialect name which will limit rendering of this prefix to only that dialect.

  `return_defaults`(*cols)
    inherited from the `return_defaults()` method of `ValuesBase`

    Make use of a RETURNING clause for the purpose of fetching server-side expressions and defaults.

    E.g.:

        stmt = table.insert().values(data='newdata').return_defaults()

        result = connection.execute(stmt)

        server_created_at = result.returned_defaults['created_at']

    When used against a backend that supports RETURNING, all column values generated by SQL expression or server-side-default will be added to any existing RETURNING clause, provided that `UpdateBase.returning()` is not used simultaneously. The column values will then be available on the result using the `ResultProxy.returned_defaults` accessor as a dictionary, referring to values keyed to the `Column` object as well as its `.key`.

    This method differs from `UpdateBase.returning()` in these ways:

    1. `ValuesBase.return_defaults()` is only intended for use with an INSERT or an UPDATE statement that matches exactly one row. While the RETURNING construct in the general sense supports multiple rows for a multi-row UPDATE or DELETE statement, or for special cases of INSERT that return multiple rows (e.g. INSERT from SELECT, multi-valued VALUES clause), `ValuesBase.return_defaults()` is intended only for an “ORM-style” single-row INSERT/UPDATE statement. The row returned by the statement is also consumed implicitly when `ValuesBase.return_defaults()` is used. By contrast, `UpdateBase.returning()` leaves the RETURNING result-set intact with a collection of any number of rows.
    2. It is compatible with the existing logic to fetch auto-generated primary key values, also known as “implicit returning”. Backends that support RETURNING will automatically make use of RETURNING in order to fetch the value of newly generated primary keys; while the `UpdateBase.returning()` method circumvents this behavior, `ValuesBase.return_defaults()` leaves it intact.
    3. It can be called against any backend. Backends that don’t support RETURNING will skip the usage of the feature, rather than raising an exception. The return value of `ResultProxy.returned_defaults` will be `None`

    `ValuesBase.return_defaults()` is used by the ORM to provide an efficient implementation for the `eager_defaults` feature of `mapper()`.

    Parameters: | cols – optional list of column key names or `Column` objects. If omitted, all column expressions evaluated on the server are added to the returning list.

    New in version 0.9.0.

    See also

    `UpdateBase.returning()`

    `ResultProxy.returned_defaults`

  `returning`(*cols)
    inherited from the `returning()` method of `UpdateBase`

    Add a RETURNING or equivalent clause to this statement.

    e.g.:

        stmt = table.update().\
                  where(table.c.data == 'value').\
                  values(status='X').\
                  returning(table.c.server_flag,
                            table.c.updated_timestamp)

        for server_flag, updated_timestamp in connection.execute(stmt):
            print(server_flag, updated_timestamp)

    The given collection of column expressions should be derived from the table that is the target of the INSERT, UPDATE, or DELETE. While `Column` objects are typical, the elements can also be expressions:

        stmt = table.insert().returning(
            (table.c.first_name + " " + table.c.last_name).
            label('fullname'))

    Upon compilation, a RETURNING clause, or database equivalent, will be rendered within the statement. For INSERT and UPDATE, the values are the newly inserted/updated values. For DELETE, the values are those of the rows which were deleted.

    Upon execution, the values of the columns to be returned are made available via the result set and can be iterated using `ResultProxy.fetchone()` and similar. For DBAPIs which do not natively support returning values (i.e. cx_oracle), SQLAlchemy will approximate this behavior at the result level so that a reasonable amount of behavioral neutrality is provided.

    Note that not all databases/DBAPIs support RETURNING. For those backends with no support, an exception is raised upon compilation and/or execution. For those who do support it, the functionality across backends varies greatly, including restrictions on executemany() and other statements which return multiple rows. Please read the documentation notes for the database in use in order to determine the availability of RETURNING.

    See also

    `ValuesBase.return_defaults()` - an alternative method tailored towards efficient fetching of server-side defaults and triggers for single-row INSERTs or UPDATEs.

  `scalar`(*multiparams, **params)
    inherited from the `scalar()` method of `Executable`

    Compile and execute this `Executable`, returning the result’s scalar representation.

  `self_group`(against=None)
    inherited from the `self_group()` method of `ClauseElement`

    Apply a ‘grouping’ to this `ClauseElement`.

    This method is overridden by subclasses to return a “grouping” construct, i.e. parenthesis. In particular it’s used by “binary” expressions to provide a grouping around themselves when placed into a larger expression, as well as by `select()` constructs when placed into the FROM clause of another `select()`. (Note that subqueries should be normally created using the `Select.alias()` method, as many platforms require nested SELECT statements to be named).

    As expressions are composed together, the application of `self_group()` is automatic - end-user code should never need to use this method directly. Note that SQLAlchemy’s clause constructs take operator precedence into account - so parenthesis might not be needed, for example, in an expression like `x OR (y AND z)` - AND takes precedence over OR.

    The base `self_group()` method of `ClauseElement` just returns self.

  `unique_params`(*optionaldict, **kwargs)
    inherited from the `unique_params()` method of `ClauseElement`

    Return a copy with `bindparam()` elements replaced.

    Same functionality as `params()`, except adds unique=True to affected bind parameters so that multiple statements can be used.

  `values`(*args, **kwargs)
    inherited from the `values()` method of `ValuesBase`

    specify a fixed VALUES clause for an INSERT statement, or the SET clause for an UPDATE.

    Note that the `Insert` and `Update` constructs support per-execution time formatting of the VALUES and/or SET clauses, based on the arguments passed to `Connection.execute()`. However, the `ValuesBase.values()` method can be used to “fix” a particular set of parameters into the statement.

    Multiple calls to `ValuesBase.values()` will produce a new construct, each one with the parameter list modified to include the new parameters sent. In the typical case of a single dictionary of parameters, the newly passed keys will replace the same keys in the previous construct. In the case of a list-based “multiple values” construct, each new list of values is extended onto the existing list of values.

    - Parameters: | **kwargs – key value pairs representing the string key of a `Column` mapped to the value to be rendered into the VALUES or SET clause:
          users.insert().values(name="some name")

          users.update().where(users.c.id==5).values(name="some name")
    - *args – As an alternative to passing key/value parameters, a dictionary, tuple, or list of dictionaries or tuples can be passed as a single positional argument in order to form the VALUES or SET clause of the statement. The forms that are accepted vary based on whether this is an `Insert` or an `Update` construct. For either an `Insert` or `Update` construct, a single dictionary can be passed, which works the same as that of the kwargs form:
          users.insert().values({"name": "some name"})

          users.update().values({"name": "some new name"})
      Also for either form but more typically for the `Insert` construct, a tuple that contains an entry for every column in the table is also accepted:
          users.insert().values((5, "some name"))
      The `Insert` construct also supports being passed a list of dictionaries or full-table-tuples, which on the server will render the less common SQL syntax of “multiple values” - this syntax is supported on backends such as SQLite, Postgresql, MySQL, but not necessarily others:
          users.insert().values([
                              {"name": "some name"},
                              {"name": "some other name"},
                              {"name": "yet another name"},
                          ])
      The above form would render a multiple VALUES statement similar to:
          INSERT INTO users (name) VALUES
                          (:name_1),
                          (:name_2),
                          (:name_3)
    It is essential to note that passing multiple values is NOT the same as using traditional executemany() form. The above syntax is a special syntax not typically used. To emit an INSERT statement against multiple rows, the normal method is to pass a multiple values list to the `Connection.execute()` method, which is supported by all database backends and is generally more efficient for a very large number of parameters. See also Executing Multiple Statements - an introduction to the traditional Core method of multiple parameter set invocation for INSERTs and other statements. Changed in version 1.0.0: an INSERT that uses a multiple-VALUES clause, even a list of length one, implies that the `Insert.inline` flag is set to True, indicating that the statement will not attempt to fetch the “last inserted primary key” or other defaults. The statement deals with an arbitrary number of rows, so the `ResultProxy.inserted_primary_key` accessor does not apply. Changed in version 1.0.0: A multiple-VALUES INSERT now supports columns with Python side default values and callables in the same way as that of an “executemany” style of invocation; the callable is invoked for each row. See Python-side defaults invoked for each row invidually when using a multivalued insert for other details. The `Update` construct supports a special form which is a list of 2-tuples, which when provided must be passed in conjunction with the `preserve_parameter_order` parameter. This form causes the UPDATE statement to render the SET clauses using the order of parameters given to `Update.values()`, rather than the ordering of columns given in the `Table`. New in version 1.0.10: - added support for parameter-ordered UPDATE statements via the `preserve_parameter_order` flag. See also Parameter-Ordered Updates - full example of the `preserve_parameter_order` flag

    See also

    Inserts, Updates and Deletes - SQL Expression Language Tutorial

    `insert()` - produce an `INSERT` statement

    `update()` - produce an `UPDATE` statement

  `where`(whereclause)

    return a new update() construct with the given expression added to its WHERE clause, joined to the existing clause via AND, if any.

  `with_hint`(text, selectable=None, dialect_name='*')
    inherited from the `with_hint()` method of `UpdateBase`

    Add a table hint for a single table to this INSERT/UPDATE/DELETE statement.

    Note

    `UpdateBase.with_hint()` currently applies only to Microsoft SQL Server. For MySQL INSERT/UPDATE/DELETE hints, use `UpdateBase.prefix_with()`.

    The text of the hint is rendered in the appropriate location for the database backend in use, relative to the `Table` that is the subject of this statement, or optionally to that of the given `Table` passed as the `selectable` argument.

    The `dialect_name` option will limit the rendering of a particular hint to a particular backend. Such as, to add a hint that only takes effect for SQL Server:

        mytable.insert().with_hint("WITH (PAGLOCK)", dialect_name="mssql")

    New in version 0.7.6.

    Parameters: | text – Text of the hint. selectable – optional `Table` that specifies an element of the FROM clause within an UPDATE or DELETE to be the subject of the hint - applies only to certain backends. dialect_name – defaults to `*`, if specified as the name of a particular dialect, will apply these hints only when that dialect is in use.

class `sqlalchemy.sql.expression.` `UpdateBase`

  Bases: `sqlalchemy.sql.base.DialectKWArgs`, `sqlalchemy.sql.expression.HasPrefixes`, `sqlalchemy.sql.expression.Executable`, `sqlalchemy.sql.expression.ClauseElement`

  Form the base for `INSERT`, `UPDATE`, and `DELETE` statements.

  `__init__`
    inherited from the `__init__` attribute of `object`

    x.__init__(...) initializes x; see help(type(x)) for signature

  `argument_for`(dialect_name, argument_name, default)
    inherited from the `argument_for()` method of `DialectKWArgs`

    Add a new kind of dialect-specific keyword argument for this class.

    E.g.:

        Index.argument_for("mydialect", "length", None)

        some_index = Index('a', 'b', mydialect_length=5)

    The `DialectKWArgs.argument_for()` method is a per-argument way adding extra arguments to the `DefaultDialect.construct_arguments` dictionary. This dictionary provides a list of argument names accepted by various schema-level constructs on behalf of a dialect.

    New dialects should typically specify this dictionary all at once as a data member of the dialect class. The use case for ad-hoc addition of argument names is typically for end-user code that is also using a custom compilation scheme which consumes the additional arguments.

    Parameters: | dialect_name – name of a dialect. The dialect must be locatable, else a `NoSuchModuleError` is raised. The dialect must also include an existing `DefaultDialect.construct_arguments` collection, indicating that it participates in the keyword-argument validation and default system, else `ArgumentError` is raised. If the dialect does not include this collection, then any keyword argument can be specified on behalf of this dialect already. All dialects packaged within SQLAlchemy include this collection, however for third party dialects, support may vary. argument_name – name of the parameter. default – default value of the parameter.

    New in version 0.9.4.

  `bind`

    Return a ‘bind’ linked to this `UpdateBase` or a `Table` associated with it.

  `compare`(other, **kw)
    inherited from the `compare()` method of `ClauseElement`

    Compare this ClauseElement to the given ClauseElement.

    Subclasses should override the default behavior, which is a straight identity comparison.

    **kw are arguments consumed by subclass compare() methods and may be used to modify the criteria for comparison. (see `ColumnElement`)

  `compile`(bind=None, dialect=None, **kw)
    inherited from the `compile()` method of `ClauseElement`

    Compile this SQL expression.

    The return value is a `Compiled` object. Calling `str()` or `unicode()` on the returned value will yield a string representation of the result. The `Compiled` object also can return a dictionary of bind parameter names and values using the `params` accessor.

    - Parameters: | bind – An `Engine` or `Connection` from which a `Compiled` will be acquired. This argument takes precedence over this `ClauseElement`‘s bound engine, if any. column_keys – Used for INSERT and UPDATE statements, a list of column names which should be present in the VALUES clause of the compiled statement. If `None`, all columns from the target table object are rendered. dialect – A `Dialect` instance from which a `Compiled` will be acquired. This argument takes precedence over the bind argument as well as this `ClauseElement`‘s bound engine, if any. inline – Used for INSERT statements, for a dialect which does not support inline retrieval of newly generated primary key columns, will force the expression used to create the new primary key value to be rendered inline within the INSERT statement’s VALUES clause. This typically refers to Sequence execution but may also refer to any server-side default generation function associated with a primary key Column. compile_kwargs – optional dictionary of additional parameters that will be passed through to the compiler within all “visit” methods. This allows any custom flag to be passed through to a custom compilation construct, for example. It is also used for the case of passing the `literal_binds` flag through:
          from sqlalchemy.sql import table, column, select

          t = table('t', column('x'))

          s = select([t]).where(t.c.x == 5)

          print s.compile(compile_kwargs={"literal_binds": True})
    New in version 0.9.0.

    See also

    How do I render SQL expressions as strings, possibly with bound parameters inlined?

  `dialect_kwargs`
    inherited from the `dialect_kwargs` attribute of `DialectKWArgs`

    A collection of keyword arguments specified as dialect-specific options to this construct.

    The arguments are present here in their original `<dialect>_<kwarg>` format. Only arguments that were actually passed are included; unlike the `DialectKWArgs.dialect_options` collection, which contains all options known by this dialect including defaults.

    The collection is also writable; keys are accepted of the form `<dialect>_<kwarg>` where the value will be assembled into the list of options.

    New in version 0.9.2.

    Changed in version 0.9.4: The `DialectKWArgs.dialect_kwargs` collection is now writable.

    See also

    `DialectKWArgs.dialect_options` - nested dictionary form

  `dialect_options`
    inherited from the `dialect_options` attribute of `DialectKWArgs`

    A collection of keyword arguments specified as dialect-specific options to this construct.

    This is a two-level nested registry, keyed to `<dialect_name>` and `<argument_name>`. For example, the `postgresql_where` argument would be locatable as:

        arg = my_object.dialect_options['postgresql']['where']

    New in version 0.9.2.

    See also

    `DialectKWArgs.dialect_kwargs` - flat dictionary form

  `execute`(*multiparams, **params)
    inherited from the `execute()` method of `Executable`

    Compile and execute this `Executable`.

  `execution_options`(**kw)
    inherited from the `execution_options()` method of `Executable`

    Set non-SQL options for the statement which take effect during execution.

    Execution options can be set on a per-statement or per `Connection` basis. Additionally, the `Engine` and ORM `Query` objects provide access to execution options which they in turn configure upon connections.

    The `execution_options()` method is generative. A new instance of this statement is returned that contains the options:

        statement = select([table.c.x, table.c.y])
        statement = statement.execution_options(autocommit=True)

    Note that only a subset of possible execution options can be applied to a statement - these include “autocommit” and “stream_results”, but not “isolation_level” or “compiled_cache”. See `Connection.execution_options()` for a full list of possible options.

    See also

    `Connection.execution_options()`

    `Query.execution_options()`

  `get_children`(**kwargs)
    inherited from the `get_children()` method of `ClauseElement`

    Return immediate child elements of this `ClauseElement`.

    This is used for visit traversal.

    **kwargs may contain flags that change the collection that is returned, for example to return a subset of items in order to cut down on larger traversals, or to return child items from a different context (such as schema-level collections instead of clause-level).

  `kwargs`
    inherited from the `kwargs` attribute of `DialectKWArgs`

    A synonym for `DialectKWArgs.dialect_kwargs`.

  `params`(*arg, **kw)

    Set the parameters for the statement.

    This method raises `NotImplementedError` on the base class, and is overridden by `ValuesBase` to provide the SET/VALUES clause of UPDATE and INSERT.

  `prefix_with`(*expr, **kw)
    inherited from the `prefix_with()` method of `HasPrefixes`

    Add one or more expressions following the statement keyword, i.e. SELECT, INSERT, UPDATE, or DELETE. Generative.

    This is used to support backend-specific prefix keywords such as those provided by MySQL.

    E.g.:

        stmt = table.insert().prefix_with("LOW_PRIORITY", dialect="mysql")

    Multiple prefixes can be specified by multiple calls to `prefix_with()`.

    Parameters: | *expr – textual or `ClauseElement` construct which will be rendered following the INSERT, UPDATE, or DELETE keyword. **kw – A single keyword ‘dialect’ is accepted. This is an optional string dialect name which will limit rendering of this prefix to only that dialect.

  `returning`(*cols)

    Add a RETURNING or equivalent clause to this statement.

    e.g.:

        stmt = table.update().\
                  where(table.c.data == 'value').\
                  values(status='X').\
                  returning(table.c.server_flag,
                            table.c.updated_timestamp)

        for server_flag, updated_timestamp in connection.execute(stmt):
            print(server_flag, updated_timestamp)

    The given collection of column expressions should be derived from the table that is the target of the INSERT, UPDATE, or DELETE. While `Column` objects are typical, the elements can also be expressions:

        stmt = table.insert().returning(
            (table.c.first_name + " " + table.c.last_name).
            label('fullname'))

    Upon compilation, a RETURNING clause, or database equivalent, will be rendered within the statement. For INSERT and UPDATE, the values are the newly inserted/updated values. For DELETE, the values are those of the rows which were deleted.

    Upon execution, the values of the columns to be returned are made available via the result set and can be iterated using `ResultProxy.fetchone()` and similar. For DBAPIs which do not natively support returning values (i.e. cx_oracle), SQLAlchemy will approximate this behavior at the result level so that a reasonable amount of behavioral neutrality is provided.

    Note that not all databases/DBAPIs support RETURNING. For those backends with no support, an exception is raised upon compilation and/or execution. For those who do support it, the functionality across backends varies greatly, including restrictions on executemany() and other statements which return multiple rows. Please read the documentation notes for the database in use in order to determine the availability of RETURNING.

    See also

    `ValuesBase.return_defaults()` - an alternative method tailored towards efficient fetching of server-side defaults and triggers for single-row INSERTs or UPDATEs.

  `scalar`(*multiparams, **params)
    inherited from the `scalar()` method of `Executable`

    Compile and execute this `Executable`, returning the result’s scalar representation.

  `self_group`(against=None)
    inherited from the `self_group()` method of `ClauseElement`

    Apply a ‘grouping’ to this `ClauseElement`.

    This method is overridden by subclasses to return a “grouping” construct, i.e. parenthesis. In particular it’s used by “binary” expressions to provide a grouping around themselves when placed into a larger expression, as well as by `select()` constructs when placed into the FROM clause of another `select()`. (Note that subqueries should be normally created using the `Select.alias()` method, as many platforms require nested SELECT statements to be named).

    As expressions are composed together, the application of `self_group()` is automatic - end-user code should never need to use this method directly. Note that SQLAlchemy’s clause constructs take operator precedence into account - so parenthesis might not be needed, for example, in an expression like `x OR (y AND z)` - AND takes precedence over OR.

    The base `self_group()` method of `ClauseElement` just returns self.

  `unique_params`(*optionaldict, **kwargs)
    inherited from the `unique_params()` method of `ClauseElement`

    Return a copy with `bindparam()` elements replaced.

    Same functionality as `params()`, except adds unique=True to affected bind parameters so that multiple statements can be used.

  `with_hint`(text, selectable=None, dialect_name='*')

    Add a table hint for a single table to this INSERT/UPDATE/DELETE statement.

    Note

    `UpdateBase.with_hint()` currently applies only to Microsoft SQL Server. For MySQL INSERT/UPDATE/DELETE hints, use `UpdateBase.prefix_with()`.

    The text of the hint is rendered in the appropriate location for the database backend in use, relative to the `Table` that is the subject of this statement, or optionally to that of the given `Table` passed as the `selectable` argument.

    The `dialect_name` option will limit the rendering of a particular hint to a particular backend. Such as, to add a hint that only takes effect for SQL Server:

        mytable.insert().with_hint("WITH (PAGLOCK)", dialect_name="mssql")

    New in version 0.7.6.

    Parameters: | text – Text of the hint. selectable – optional `Table` that specifies an element of the FROM clause within an UPDATE or DELETE to be the subject of the hint - applies only to certain backends. dialect_name – defaults to `*`, if specified as the name of a particular dialect, will apply these hints only when that dialect is in use.

class `sqlalchemy.sql.expression.` `ValuesBase`(table, values, prefixes)

  Bases: `sqlalchemy.sql.expression.UpdateBase`

  Supplies support for `ValuesBase.values()` to INSERT and UPDATE constructs.

  `return_defaults`(*cols)

    Make use of a RETURNING clause for the purpose of fetching server-side expressions and defaults.

    E.g.:

        stmt = table.insert().values(data='newdata').return_defaults()

        result = connection.execute(stmt)

        server_created_at = result.returned_defaults['created_at']

    When used against a backend that supports RETURNING, all column values generated by SQL expression or server-side-default will be added to any existing RETURNING clause, provided that `UpdateBase.returning()` is not used simultaneously. The column values will then be available on the result using the `ResultProxy.returned_defaults` accessor as a dictionary, referring to values keyed to the `Column` object as well as its `.key`.

    This method differs from `UpdateBase.returning()` in these ways:

    1. `ValuesBase.return_defaults()` is only intended for use with an INSERT or an UPDATE statement that matches exactly one row. While the RETURNING construct in the general sense supports multiple rows for a multi-row UPDATE or DELETE statement, or for special cases of INSERT that return multiple rows (e.g. INSERT from SELECT, multi-valued VALUES clause), `ValuesBase.return_defaults()` is intended only for an “ORM-style” single-row INSERT/UPDATE statement. The row returned by the statement is also consumed implicitly when `ValuesBase.return_defaults()` is used. By contrast, `UpdateBase.returning()` leaves the RETURNING result-set intact with a collection of any number of rows.
    2. It is compatible with the existing logic to fetch auto-generated primary key values, also known as “implicit returning”. Backends that support RETURNING will automatically make use of RETURNING in order to fetch the value of newly generated primary keys; while the `UpdateBase.returning()` method circumvents this behavior, `ValuesBase.return_defaults()` leaves it intact.
    3. It can be called against any backend. Backends that don’t support RETURNING will skip the usage of the feature, rather than raising an exception. The return value of `ResultProxy.returned_defaults` will be `None`

    `ValuesBase.return_defaults()` is used by the ORM to provide an efficient implementation for the `eager_defaults` feature of `mapper()`.

    Parameters: | cols – optional list of column key names or `Column` objects. If omitted, all column expressions evaluated on the server are added to the returning list.

    New in version 0.9.0.

    See also

    `UpdateBase.returning()`

    `ResultProxy.returned_defaults`

  `values`(*args, **kwargs)

    specify a fixed VALUES clause for an INSERT statement, or the SET clause for an UPDATE.

    Note that the `Insert` and `Update` constructs support per-execution time formatting of the VALUES and/or SET clauses, based on the arguments passed to `Connection.execute()`. However, the `ValuesBase.values()` method can be used to “fix” a particular set of parameters into the statement.

    Multiple calls to `ValuesBase.values()` will produce a new construct, each one with the parameter list modified to include the new parameters sent. In the typical case of a single dictionary of parameters, the newly passed keys will replace the same keys in the previous construct. In the case of a list-based “multiple values” construct, each new list of values is extended onto the existing list of values.

    - Parameters: | **kwargs – key value pairs representing the string key of a `Column` mapped to the value to be rendered into the VALUES or SET clause:
          users.insert().values(name="some name")

          users.update().where(users.c.id==5).values(name="some name")
    - *args – As an alternative to passing key/value parameters, a dictionary, tuple, or list of dictionaries or tuples can be passed as a single positional argument in order to form the VALUES or SET clause of the statement. The forms that are accepted vary based on whether this is an `Insert` or an `Update` construct. For either an `Insert` or `Update` construct, a single dictionary can be passed, which works the same as that of the kwargs form:
          users.insert().values({"name": "some name"})

          users.update().values({"name": "some new name"})
      Also for either form but more typically for the `Insert` construct, a tuple that contains an entry for every column in the table is also accepted:
          users.insert().values((5, "some name"))
      The `Insert` construct also supports being passed a list of dictionaries or full-table-tuples, which on the server will render the less common SQL syntax of “multiple values” - this syntax is supported on backends such as SQLite, Postgresql, MySQL, but not necessarily others:
          users.insert().values([
                              {"name": "some name"},
                              {"name": "some other name"},
                              {"name": "yet another name"},
                          ])
      The above form would render a multiple VALUES statement similar to:
          INSERT INTO users (name) VALUES
                          (:name_1),
                          (:name_2),
                          (:name_3)
    It is essential to note that passing multiple values is NOT the same as using traditional executemany() form. The above syntax is a special syntax not typically used. To emit an INSERT statement against multiple rows, the normal method is to pass a multiple values list to the `Connection.execute()` method, which is supported by all database backends and is generally more efficient for a very large number of parameters. See also Executing Multiple Statements - an introduction to the traditional Core method of multiple parameter set invocation for INSERTs and other statements. Changed in version 1.0.0: an INSERT that uses a multiple-VALUES clause, even a list of length one, implies that the `Insert.inline` flag is set to True, indicating that the statement will not attempt to fetch the “last inserted primary key” or other defaults. The statement deals with an arbitrary number of rows, so the `ResultProxy.inserted_primary_key` accessor does not apply. Changed in version 1.0.0: A multiple-VALUES INSERT now supports columns with Python side default values and callables in the same way as that of an “executemany” style of invocation; the callable is invoked for each row. See Python-side defaults invoked for each row invidually when using a multivalued insert for other details. The `Update` construct supports a special form which is a list of 2-tuples, which when provided must be passed in conjunction with the `preserve_parameter_order` parameter. This form causes the UPDATE statement to render the SET clauses using the order of parameters given to `Update.values()`, rather than the ordering of columns given in the `Table`. New in version 1.0.10: - added support for parameter-ordered UPDATE statements via the `preserve_parameter_order` flag. See also Parameter-Ordered Updates - full example of the `preserve_parameter_order` flag

    See also

    Inserts, Updates and Deletes - SQL Expression Language Tutorial

    `insert()` - produce an `INSERT` statement

    `update()` - produce an `UPDATE` statement